local path = (...):gsub("[^%.]*$", "")
local Inky = require("3rd.inky")
local Button = require(path .. "button")
local Checkbox = require(path .. "checkbox")
local Slider = require(path .. "slider")
local nineBgmenu
nineBgmenu = require(path .. "nine").nineBgmenu
local g = require("global")
local scene = Inky.scene()
local pointer = Inky.pointer(scene)
local wait_for_input
local buttons = {
  skip = Button.new(scene),
  fullscreen = Button.new(scene),
  cutscene = Button.new(scene),
  back = Button.new(scene)
}
buttons.skip.props.text = g.key.skip:upper()
buttons.fullscreen.props.text = g.key.fullscreen:upper()
buttons.cutscene.props.text = g.key.cutscene:upper()
buttons.back.props.text = function()
  return TEXT(333, "Return to game")
end
buttons.skip.props.callback = function()
  wait_for_input = "skip"
end
buttons.fullscreen.props.callback = function()
  wait_for_input = "fullscreen"
end
buttons.cutscene.props.callback = function()
  wait_for_input = "cutscene"
end
buttons.back.props.callback = function()
  local gui = require("gui")
  return gui:set_menu(not g.game_started and "start" or nil)
end
local checkbox1 = Checkbox.new(scene)
checkbox1.props.checked = not not g.use_hires_font
checkbox1.props.callback = function(value)
  g.use_hires_font = value
end
local sliders = {
  volume = Slider.new(scene)
}
local audio = require("audio")
sliders.volume.props.progress = g.volume
sliders.volume.props.callback = function(progress)
  return audio.set_volume(progress)
end
local M = { }
M.update = function(self)
  if not wait_for_input then
    return pointer:setPosition(g.p.x, g.p.y)
  end
end
local menu_padding = 11
local menu_width = g.game_width - 2 * menu_padding
local menu_height = g.game_height - 2 * menu_padding
local button_width = 100
local newline
newline = function(x, y)
  y = y + g.font:getHeight()
  love.graphics.setColor(0.694, 0.318, 0.055)
  love.graphics.rectangle("fill", x, y, menu_width, 1)
  love.graphics.setColor(1, 1, 1)
  y = y + 1
  return x, y
end
local print_text
print_text = function(text, x, y)
  if g.use_hires_font then
    love.graphics.setShader(g.hires_shader_inner)
  end
  love.graphics.print(text, x, y, 0, g.use_hires_font and g.hires_scale or 1)
  if g.use_hires_font then
    return love.graphics.setShader()
  end
end
local printf_text
printf_text = function(text, x, y, w, align)
  local font_scale = g.use_hires_font and g.hires_scale or 1
  if g.use_hires_font then
    love.graphics.setShader(g.hires_shader_inner)
  end
  love.graphics.printf(text, x, y, w / font_scale, align, 0, font_scale)
  if g.use_hires_font then
    return love.graphics.setShader()
  end
end
M.draw = function(self)
  love.graphics.push("all")
  love.graphics.setFont(g.use_hires_font and g.font_hires or g.font_no_outline)
  local font_scale = g.use_hires_font and g.hires_scale or 1
  love.graphics.scale(g.scale)
  love.graphics.translate(g.xshift, g.yshift)
  love.graphics.setColor(0, 0, 0, 0.2)
  love.graphics.rectangle("fill", 0, 0, g.game_width, g.game_height)
  love.graphics.setColor(1, 1, 1)
  local x, y = menu_padding, menu_padding
  nineBgmenu:draw(x, y, menu_width, menu_height)
  y = y + 5
  scene:beginFrame()
  print_text(TEXT(334, "High-resolution text"), x, y)
  checkbox1:render(x + menu_width - Checkbox.size.x, y + Checkbox.yoffset, Checkbox.size.x, Checkbox.size.y)
  x, y = newline(x, y)
  print_text(TEXT(335, "Music volume"), x, y)
  sliders.volume:render(x + menu_width - button_width, y + Slider.yoffset, button_width, Slider.height)
  x, y = newline(x, y)
  print_text(TEXT(336, "Button to skip text"), x, y)
  buttons.skip:render(x + menu_width - button_width, y, button_width, Button.height)
  x, y = newline(x, y)
  print_text(TEXT(337, "Button to toggle fullscreen"), x, y)
  buttons.fullscreen:render(x + menu_width - button_width, y, button_width, Button.height)
  x, y = newline(x, y)
  print_text(TEXT(338, "Button to skip cutscenes"), x, y)
  buttons.cutscene:render(x + menu_width - button_width, y, button_width, Button.height)
  x, y = newline(x, y)
  local text = buttons.back.props.text()
  local w = love.graphics.getFont():getWidth(text) * font_scale + 10
  buttons.back:render(x + (menu_width - w) / 2, menu_padding + menu_height - Button.height, w, Button.height)
  do
    local k = wait_for_input
    if k then
      love.graphics.setColor(0, 0, 0, 0.5)
      love.graphics.rectangle("fill", 0, 0, g.game_width, g.game_height)
      love.graphics.setColor(1, 1, 1)
      x, y = menu_padding + 30, menu_padding + 40
      nineBgmenu:draw(x, y, menu_width - 60, menu_height - 80)
      if k == "skip" then
        text = TEXT(339, "Press a key or one of the mouse buttons to change the previous binding.")
      else
        text = TEXT(340, "Press a key to change the previous binding.")
      end
      printf_text(text, x, y, menu_width - 60, "left")
    end
  end
  scene:finishFrame()
  return love.graphics.pop()
end
local skip_next_release
M.mousereleased = function(self)
  if skip_next_release then
    skip_next_release = false
    return 
  end
  return pointer:raise("release")
end
M.mousepressed = function(self, button)
  if wait_for_input == "skip" then
    g.key.skip = button
    g.skip_with_mouse = true
    local _exp_0 = button
    if 1 == _exp_0 then
      buttons.skip.props.text = function()
        return TEXT(341, "Mouse left")
      end
    elseif 2 == _exp_0 then
      buttons.skip.props.text = function()
        return TEXT(342, "Mouse right")
      end
    elseif 3 == _exp_0 then
      buttons.skip.props.text = function()
        return TEXT(343, "Mouse middle")
      end
    else
      buttons.skip.props.text = "Mouse " .. button
    end
    wait_for_input = nil
    skip_next_release = true
    return true
  else
    return pointer:raise("press")
  end
end
M.keypressed = function(self, key)
  do
    local k = wait_for_input
    if k then
      g.key[k] = key
      buttons[k].props.text = key:upper()
      wait_for_input = nil
      if k == "skip" then
        g.skip_with_mouse = false
      end
      return true
    end
  end
end
M.mousemoved = function(self)
  if love.mouse.isDown(1) then
    return pointer:raise("drag")
  end
end
return M
