local path = (...):gsub(".init$", "") .. '.'
local g = require("global")
local Atlas = require("engine.steelpan.atlas")
g.interface_atlas = Atlas("assets/spritesheets/interface.lua")
local lc = require("engine")
local skip = require("skip")
g.font_lowres = love.graphics.setNewFont("assets/fonts/uncle_lee_6x3_outline.fnt")
g.font_no_outline = love.graphics.newFont("assets/fonts/uncle_lee_6x3.fnt")
g.font_hires = love.graphics.newFont("assets/fonts/hires.fnt")
g.font_hires:setFilter("linear", "linear")
g.font = g.font_lowres
g.hires_scale = g.font_lowres:getHeight() / g.font_hires:getHeight()
g.hires_shader_inner = love.graphics.newShader([[const float smoothing = 2.0/16.0;
const float outerEdgeCenter = 0.7;

vec4 effect(vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords) {
    float distance = Texel(tex, texture_coords).a;
    float alpha = smoothstep(outerEdgeCenter - smoothing, outerEdgeCenter + smoothing, distance);
    return vec4(color.rgb, color.a * alpha);
}
]])
g.hires_shader_outer = love.graphics.newShader([[const float smoothing = 1.0/16.0;
const float outlineWidth = 8.0/16.0;
const float outerEdgeCenter = 0.7 - outlineWidth;
const vec3 outlineColor = vec3(0.0, 0.0, 0.0);

vec4 effect(vec4 color, Image tex, vec2 texture_coords, vec2 screen_coords) {
    float distance = Texel(tex, texture_coords).a;
    float alpha = smoothstep(outerEdgeCenter - smoothing, outerEdgeCenter + smoothing, distance);
    return vec4(outlineColor, alpha);
}
]])
local cursor = require(path .. "cursor")
local look = require(path .. "look")
local inventory = require(path .. "inventory")
local dialogues = require(path .. "dialogues")
local cog = require(path .. "cog")
local M = { }
M.draw_canvas = function(self)
  if lc.dialogues:is_running() then
    dialogues:draw_canvas()
  else
    inventory:draw()
    if not require("gui").menu then
      look:draw_canvas()
    end
  end
  skip:draw_canvas()
  return cog:draw_canvas()
end
M.draw_screen = function(self)
  if lc.dialogues:is_running() then
    dialogues:draw_screen()
  else
    if not require("gui").menu then
      look:draw_screen()
    end
  end
  return cursor:draw()
end
M.update = function(self, dt)
  look:update()
  if lc.dialogues:is_running() then
    return dialogues:update()
  end
end
M.mousemoved = function(self, x, y, p)
  cursor:mousemoved(x, y)
  if lc.dialogues:is_running() then
    return dialogues:mousemoved(p.x, p.y)
  else
    return inventory:mousemoved(p.x, p.y)
  end
end
M.mousereleased = function(self)
  if lc.dialogues:is_running() then
    return dialogues:mousereleased()
  end
end
M.wheelmoved = function(self, x, y)
  if lc.dialogues:is_running() then
    return dialogues:wheelmoved(y)
  end
end
M.mouse_on_inventory = function(self, x, y)
  return inventory:mouse_on_inventory(x, y)
end
return M
